import axios from 'axios'

function addFieldsToUrl ({url, fields = []}) {
  return fields.length > 0 ? url + '?fields=pk,' + fields : url
}

function getInstanceBaseUrl () {
  if (process.env.NODE_ENV === 'production') {
    return `https://${window.location.host}${window.location.pathname}`
  } else {
    return 'http://localhost:8000/'
  }
}

let ax = axios.create({
  baseURL: getInstanceBaseUrl(),
  headers: {'Authorization': 'JWT ' + sessionStorage.getItem('token')}
})

export async function fetchJWT (credentials) {
  const token = (await ax.post('/api/get-token/', credentials)).data.token
  ax.defaults.headers['Authorization'] = `JWT ${token}`
  return token
}

export async function refreshJWT (token) {
  const newToken = (await ax.post('/api/refresh-token/', {token})).data.token
  ax.defaults.headers['Authorization'] = `JWT ${newToken}`
  return token
}

export async function fetchJWTTimeDelta () {
  return (await ax.get('/api/jwt-time-delta/')).data.timeDelta
}

export async function fetchUserRole () {
  return (await ax.get('/api/user-role/')).data.role
}

export async function fetchStudentSelfData () {
  return (await ax.get('/api/student-page/')).data
}

export async function fetchStudentSubmissions () {
  return (await ax.get('/api/student-submissions/')).data
}

export async function fetchSubmissionFeedbackTests ({pk}) {
  return (await ax.get(`/api/submission/${pk}/`)).data
}

export async function fetchAllStudents (fields = []) {
  const url = addFieldsToUrl({
    url: '/api/student/',
    fields
  })
  return (await ax.get(url)).data
}

export async function fetchStudent ({pk, fields = []}) {
  const url = addFieldsToUrl({
    url: `/api/student/${pk}/`,
    fields
  })
  return (await ax.get(url)).data
}

export async function fetchAllTutors (fields = []) {
  const url = addFieldsToUrl({
    url: '/api/tutor/',
    fields
  })
  return (await ax.get(url)).data
}

export async function fetchSubscriptions () {
  return (await ax.get('/api/subscription/')).data
}

export async function deactivateSubscription ({pk}) {
  const url = `/api/subscription/${pk}/`
  return (await ax.delete(url)).data
}

export async function fetchSubscription (subscriptionPk) {
  return (await ax.get(`/api/subscription/${subscriptionPk}/`)).data
}

export async function fetchAllFeedback (fields = []) {
  const url = addFieldsToUrl({
    url: '/api/feedback/',
    fields
  })
  return (await ax.get(url)).data
}

export async function fetchFeedback ({ofSubmission}) {
  const url = `/api/feedback/${ofSubmission}/`
  return (await ax.get(url)).data
}

export async function fetchExamType ({examPk, fields = []}) {
  const url = addFieldsToUrl({
    url: `/api/examtype/${examPk !== undefined ? examPk + '/' : ''}`,
    fields})
  return (await ax.get(url)).data
}

export async function fetchStatistics (opt = {fields: []}) {
  const url = addFieldsToUrl({
    url: '/api/statistics/',
    fields: opt.fields
  })
  return (await ax.get(url)).data
}

export async function subscribeTo (type, key, stage) {
  let data = {
    query_type: type
  }

  if (key) {
    data.query_key = key
  }
  if (stage) {
    data.feedback_stage = stage
  }

  return (await ax.post('/api/subscription/', data)).data
}

export async function createAssignment ({subscription}) {
  const data = {
    subscription: subscription.pk
  }
  return (await ax.post(`/api/assignment/`, data)).data
}

export async function submitFeedbackForAssignment ({feedback}) {
  return (await ax.post('/api/feedback/', feedback)).data
}

export async function submitUpdatedFeedback ({feedback}) {
  return (await ax.patch(`/api/feedback/${feedback.of_submission}/`, feedback)).data
}

export async function fetchSubmissionTypes (fields = []) {
  let url = '/api/submissiontype/'
  if (fields.length > 0) {
    url += '?fields=pk,' + fields
  }
  return (await ax.get(url)).data
}

export async function deleteAssignment ({assignment}) {
  const url = `/api/assignment/${assignment.pk}/`
  return (await ax.delete(url)).data
}

export async function deleteComment (comment = {pk: undefined}) {
  const url = `/api/feedback-comment/${comment.pk}/`
  return (await ax.delete(url)).data
}

export async function patchComment (comment = {pk: undefined}) {
  const url = `/api/feedback-comment/${comment.pk}/`
  return (await ax.patch(url, comment)).data
}

export async function activateAllStudentAccess () {
  return ax.post('/api/student/activate/')
}

export async function deactivateAllStudentAccess () {
  return ax.post('/api/student/deactivate/')
}

export default ax
