import {fetchJWT, fetchJWTTimeDelta, fetchUserRole, refreshJWT} from '@/api'
import gradySays from '../grady_speak'

function initialState () {
  return {
    token: sessionStorage.getItem('token'),
    lastTokenRefreshTry: Date.now(),
    refreshingToken: false,
    username: '',
    jwtTimeDelta: 0,
    userRole: '',
    message: ''
  }
}

export const authMut = Object.freeze({
  SET_MESSAGE: 'SET_MESSAGE',
  SET_JWT_TOKEN: 'SET_JWT_TOKEN',
  SET_JWT_TIME_DELTA: 'SET_JWT_TIME_DELTA',
  SET_USERNAME: 'SET_USERNAME',
  SET_USER_ROLE: 'SET_USER_ROLE',
  SET_LAST_TOKEN_REFRESH_TRY: 'SET_LAST_TOKEN_REFRESH_TRY',
  RESET_STATE: 'RESET_STATE',
  SET_REFRESHING_TOKEN: 'SET_REFRESHING_TOKEN'
})

const authentication = {
  state: initialState(),
  getters: {
    gradySpeak: () => {
      return gradySays[Math.floor(Math.random() * gradySays.length)]
    },
    isStudent: state => {
      return state.userRole === 'Student'
    },
    isTutor: state => {
      return state.userRole === 'Tutor'
    },
    isReviewer: state => {
      return state.userRole === 'Reviewer'
    },
    isTutorOrReviewer: (state, getters) => {
      return getters.isTutor || getters.isReviewer
    },
    isLoggedIn: state => !!state.token
  },
  mutations: {
    [authMut.SET_MESSAGE]: function (state, message) {
      state.message = message
    },
    [authMut.SET_JWT_TOKEN]: function (state, token) {
      sessionStorage.setItem('token', token)
      state.token = token
    },
    [authMut.SET_JWT_TIME_DELTA]: function (state, timeDelta) {
      state.jwtTimeDelta = timeDelta
    },
    [authMut.SET_USERNAME]: function (state, username) {
      state.username = username
    },
    [authMut.SET_USER_ROLE]: function (state, userRole) {
      state.userRole = userRole
    },
    [authMut.SET_REFRESHING_TOKEN]: function (state, refreshing) {
      state.refreshingToken = refreshing
    },
    [authMut.SET_LAST_TOKEN_REFRESH_TRY]: function (state) {
      state.lastTokenRefreshTry = Date.now()
    },
    [authMut.RESET_STATE]: function (state) {
      sessionStorage.setItem('token', '')
      Object.assign(state, initialState())
    }
  },
  actions: {
    async getJWT (context, credentials) {
      try {
        const token = await fetchJWT(credentials)
        context.commit(authMut.SET_USERNAME, credentials.username)
        context.commit(authMut.SET_JWT_TOKEN, token)
      } catch (error) {
        console.log(error)
        if (error.response) {
          const errorMsg = 'Unable to log in with provided credentials.'
          context.commit(authMut.SET_MESSAGE, errorMsg)
          throw errorMsg
        } else {
          const errorMsg = 'Cannot reach server.'
          context.commit(authMut.SET_MESSAGE, errorMsg)
          throw errorMsg
        }
      } finally {
        context.commit(authMut.SET_LAST_TOKEN_REFRESH_TRY)
      }
    },
    async refreshJWT ({state, commit}) {
      commit(authMut.SET_REFRESHING_TOKEN, true)
      try {
        const token = await refreshJWT(state.token)
        commit(authMut.SET_JWT_TOKEN, token)
      } finally {
        commit(authMut.SET_REFRESHING_TOKEN, false)
        commit(authMut.SET_LAST_TOKEN_REFRESH_TRY)
      }
    },
    async getUserRole ({commit}) {
      try {
        const userRole = await fetchUserRole()
        commit(authMut.SET_USER_ROLE, userRole)
      } catch (err) {
        commit(authMut.SET_MESSAGE, "You've been logged out.")
      }
    },
    async getJWTTimeDelta ({commit}) {
      try {
        const delta = await fetchJWTTimeDelta()
        // multiply by 1000 to convert to ms
        commit(authMut.SET_JWT_TIME_DELTA, delta * 1000)
      } catch (err) {
        console.log(err)
      }
    }
  }
}

export default authentication
